"""
Configuration Module.
This reads the configuration file and conveniently makes the settings available to KCG
"""
import ConfigParser
import ast
import os
import sys

error = False

class NoValueException(Exception):
    """
    Simple Exception that gets thrown when no Value was given in the configuration file
    """
    pass

def leval(string):
    """
    Wrapper arount ast.literal_eval that throws NoValueException if the value was empty
    :param string:
    :return:
    """
    if string == "":
        raise NoValueException("No value given for option")
    else:
        return ast.literal_eval(string)


def read():
    """
    This function reads the configuration file and the default config fille (in case the
    user config file does not contain all values) and adds them as parameters to this module
    :return:
    """
    global error
    config = ConfigParser.ConfigParser()
    config.optionxform = str
    config.read(os.path.expanduser("~")+"/.kcg/config.cfg")
    defaultConfig = ConfigParser.ConfigParser()
    defaultConfig.optionxform = str
    defaultConfig.read(os.path.join(os.path.dirname(__file__), "config.cfg"))

    Machine_conf = ["bunches_per_turn", "save_header", "tRev"]
    Ui_conf = ["language", "default_save_location", "default_subdirectory_name", "force_ask", "show_advanced_control"]
    Logging_conf = ["epics_test_pv", "epics_base_path"]

    error = False

    try:
        for conf in Machine_conf:
            if config.has_option("Machine", conf):
                try:
                    globals()[conf] = leval(config.get('Machine', conf))
                except NoValueException:
                    error = True
            else:
                # print "Using default configuration value for " + conf
                # globals()[conf] = leval(defaultConfig.get("Machine", conf))
                error = True
        for conf in Ui_conf:
            if config.has_option("Ui", conf):
                globals()[conf] = leval(config.get('Ui', conf))
            else:
                globals()[conf] = leval(defaultConfig.get("Ui", conf))
                print "Using default configuration value for " + conf
        for conf in Logging_conf:
            if config.has_option("Logging", conf):
                globals()[conf] = leval(config.get('Logging', conf))
            else:
                globals()[conf] = leval(defaultConfig.get("Logging", conf))
                print "Using default configuration value for " + conf

        if config.has_option("Logging", "epics_log_entry_pvs"):
            globals()["epics_log_entry_pvs"] = leval(config.get('Logging', "epics_log_entry_pvs"))
        else:
            globals()["epics_log_entry_pvs"] = leval(defaultConfig.get('Logging', "epics_log_entry_pvs"))
            print "Using default configuration value for epics_log_entry_pvs"

        if config.has_option("Logging", "default_log_entries"):
            globals()["default_log_entries"] = leval(config.get('Logging', "default_log_entries"))
        else:
            globals()["default_log_entries"] = leval(defaultConfig.get('Logging', "default_log_entries"))
            print "Using default configuration value for default_log_entries"

        # -----[ Do non standard config options ]------------
        if config.has_section('Misc'):
            for conf, val in config.items("Misc"):
                globals()[conf] = leval(val)

        for conf, val in defaultConfig.items("Misc"):
            if not conf in globals().keys():
                globals()[conf] = leval(val)

    except (ConfigParser.NoOptionError, ConfigParser.NoSectionError) as e:
        error = True
        print "There was an error parsing configuration " + str(e)


def setup():
    """
    Check for the user config file and if not exists calls doSetup
    :return:
    """
    if not os.path.isfile(os.path.expanduser("~")+"/.kcg/config.cfg"):
        doSetup()

def doSetup():
    """
    Shows the initial config dialog
    :return:
    """
    from PyQt4 import QtGui, QtCore
    from widgets.initialconfig import ConfigSetup
    setupConfigApp = QtGui.QApplication([])
    setupConfig = ConfigSetup()
    setupConfig.show()
    setupConfigApp.exec_()
    if not setupConfig.result:
        sys.exit(122)
    global error
    error = False
    read()

read()
