"""
This Module Is the Acquiresettings subWindow.
"""
import sys

from PyQt4 import QtGui, QtCore

from ..base import kcgwidget as kcgw
from ..base.backend import board
#bif = sys.modules["base.backendinterface"] # reuse backendinterfacemodule since importing at this point will result in an error
from ..base import backendinterface as bif
from ..base.groupedelements import Elements
from .. import config
tr = kcgw.tr

__widget_id__ = None

class acquireSettings(kcgw.KCGWidgets):
    """
    This is the actual Acquire Settings Window.
    """
    def __init__(self, unique_id, parent=None):
        """
        Initialise the acquire settings window
        :param unique_id: the id for thiw widget
        :param parent: parent object
        :return: -
        """
        super(acquireSettings, self).__init__()

        self.id = unique_id
        self.par = parent
        self.layout = QtGui.QGridLayout()
        self.outerLayout = QtGui.QVBoxLayout()
        self.outerLayout.addLayout(self.layout)
        self.outerLayout.addStretch(1)
        self.setLayout(self.outerLayout)

        #---------[ Create Labels and corresponding Fields ]---------
        self.numOfOrbitsLabel = self.createLabel(tr("Label", "Number of orbits to observe")) # TODO: Better Text
        self.numOfSkipOrbitsLabel = self.createLabel(tr("Label", "Number of orbits to skip")) # TODO: Better Text
        self.numOfOrbitsSpinbox = self.createSpinbox(1, 10000000, connect=self.on_number_of_orbits_changed) # TODO: does 10000000 as max make sense?
        self.numOfSkipOrbitsSpinbox = self.createSpinbox(0, 100, connect=self.on_number_of_skipped_orbits_changed) # TODO: 100 as max make sense?
        self.countLabel = self.createLabel(tr("Label", "Number of acquisitions"))
        self.countSpinbox = self.createSpinbox(1, 10000000, start_value=10, connect=self.on_count_changed) # TODO: does 10000000 as max make sense?
        self.waitLabel = self.createLabel(tr("Label", "Wait(s)"))
        self.waitSpinbox = self.createSpinbox(0, 60, start_value=15, connect=self.on_wait_changed) # TODO: does 60 as max and default as 15 make sense?
        self.simulatePilotBunch = self.createCheckbox(tr("Button", "Simulate Pilot Bunch"), connect=self.on_simulate_pilot_bunch_changed)
        self.buildSpectrogrammTickbox = self.createCheckbox(tr("Button", "Build Spectograms"), connect=self.on_build_spectrograms_changed)

        # ----------[ Trigger part ]--------------------
        self.trigger_skip_label = self.createLabel(tr("Label", "Number of trigger signals to skip"))
        self.trigger_skip_signals = self.createSpinbox(0, 1000, 1, start_value=0, connect=self.on_trigger_skip_signals_changed)
        self.timeout_label = self.createLabel(tr("Label", "Timeout (s)"))
        self.timeout = self.createSpinbox(0, 10000000, start_value=12, connect=self.on_trigger_timeout_changed)
        self.method_label = self.createLabel(tr("Label", "Method"))
        self.method = kcgw.Switch("1", "2")
        self.method.clicked.connect(self.on_trigger_method_changed)
        self.trigger_skip_label.hide()
        self.trigger_skip_signals.hide()
        self.timeout_label.hide()
        self.timeout.hide()
        self.method_label.hide()
        self.method.hide()

        self.trigger_tick = self.createCheckbox(tr("Label", "Use External Trigger"), connect=self.use_external_trigger)

        # -------[ register observers ]------
        board.config.observe(self.numOfOrbitsSpinbox, lambda x: self.setValueSilent(self.numOfOrbitsSpinbox, x), 'orbits_observe')
        board.config.observe(self.numOfSkipOrbitsSpinbox, lambda x: self.setValueSilent(self.numOfSkipOrbitsSpinbox, x), 'orbits_skip')
        board.config.observe(self.countSpinbox, lambda x: self.setValueSilent(self.countSpinbox, x), 'acquisition_count')
        board.config.observe(self.waitSpinbox, lambda x: self.setValueSilent(self.waitSpinbox, x), 'orbits_wait_time')
        board.config.observe(self.simulatePilotBunch, lambda x: self.tickSilent(self.simulatePilotBunch, x), 'pilot_bunch')
        board.config.observe(self.buildSpectrogrammTickbox, lambda x: self.tickSilent(self.buildSpectrogrammTickbox, x), 'build_spectrograms')
        board.config.observe(self.trigger_skip_signals, lambda x: self.setValueSilent(self.trigger_skip_signals, x), 'trigger_skip')
        board.config.observe(self.timeout, lambda x: self.setValueSilent(self.timeout, x), 'trigger_timeout')
        board.config.observe(self.trigger_tick, self.update_external_trigger, 'use_trigger')
        # -------[ register elements ]-------
        Elements.addItem(["acquire", "no_board"],
                         [
                             self.numOfOrbitsSpinbox,
                             self.numOfSkipOrbitsSpinbox,
                             self.countSpinbox,
                             self.waitSpinbox,
                             self.buildSpectrogrammTickbox,
                             self.simulatePilotBunch,
                             self.trigger_skip_signals,
                             self.timeout,
                             self.method,
                             self.trigger_tick
                         ])

        # -------[ Add to grid ]-------------
        self.layout.addWidget(self.numOfOrbitsLabel, 0, 0)
        self.layout.addWidget(self.numOfOrbitsSpinbox, 0, 1)
        self.layout.addWidget(self.numOfSkipOrbitsLabel, 1, 0)
        self.layout.addWidget(self.numOfSkipOrbitsSpinbox, 1, 1)
        self.layout.addWidget(self.countLabel, 2, 0)
        self.layout.addWidget(self.countSpinbox, 2, 1)
        self.layout.addWidget(self.waitLabel, 3, 0)
        self.layout.addWidget(self.waitSpinbox, 3, 1)
        self.layout.addWidget(self.simulatePilotBunch, 9, 0)
        self.layout.addWidget(self.buildSpectrogrammTickbox, 9, 1)
        self.layout.addWidget(self.trigger_tick, 7, 0)
        line = QtGui.QFrame()
        line.setFrameShape(QtGui.QFrame.HLine)
        self.layout.addWidget(line, 8, 0, 1, 2)

        self.setWindowTitle(tr("Heading", "Acquire Settings"))
        self.setValues()

    def update_external_trigger(self, state):
        self.tickSilent(self.trigger_tick, state)
        self.use_external_trigger(state, update_config=False)

    def use_external_trigger(self, state, update_config=True):
        if state:
            if update_config:
                board.config.update('use_trigger', True)
            self.layout.removeWidget(self.waitLabel)
            self.layout.removeWidget(self.waitSpinbox)
            self.waitLabel.hide()
            self.waitSpinbox.hide()

            self.layout.addWidget(self.timeout_label, 3, 0)
            self.layout.addWidget(self.timeout, 3, 1)
            self.layout.addWidget(self.trigger_skip_label, 4, 0)
            self.layout.addWidget(self.trigger_skip_signals, 4, 1)
            self.layout.addWidget(self.method_label, 5, 0)
            self.layout.addWidget(self.method, 5, 1)
            self.timeout_label.show()
            self.timeout.show()
            self.trigger_skip_label.show()
            self.trigger_skip_signals.show()
            self.method_label.show()
            self.method.show()
        else:
            if update_config:
                board.config.update('use_trigger', False)
            self.layout.addWidget(self.waitLabel, 3, 0)
            self.layout.addWidget(self.waitSpinbox, 3, 1)
            self.waitLabel.show()
            self.waitSpinbox.show()

            self.layout.removeWidget(self.timeout_label)
            self.layout.removeWidget(self.timeout)
            self.layout.removeWidget(self.trigger_skip_label)
            self.layout.removeWidget(self.trigger_skip_signals)
            self.layout.removeWidget(self.method_label)
            self.layout.removeWidget(self.method)
            self.timeout_label.hide()
            self.timeout.hide()
            self.trigger_skip_label.hide()
            self.trigger_skip_signals.hide()
            self.method_label.hide()
            self.method.hide()
        QtCore.QCoreApplication.processEvents()
        self.parent().adjustSize()
#        self.parent().resize(self.parent().minimumSizeHint())

    def setValueSilent(self, element, value):
        """
        Set a Value silent without notifying observers
        :param element: what element
        :param value: set to what value
        :return: -
        """
        element.blockSignals(True)
        element.setValue(value)
        element.blockSignals(False)
    def tickSilent(self, element, state):
        """
        Tick a tickbox without notifying observers
        :param element: what tickbox
        :param state: (bool) to what state
        :return: -
        """
        element.blockSignals(True)
        element.setChecked(state)
        element.blockSignals(False)

    def setValues(self):
        """
        Set Initial Values when creating this Window.
        This will read the appropriate Values from the base.backend.board.BoardConfiguration instance used in the gui.
        :return: -
        """
        self.setValueSilent(self.numOfOrbitsSpinbox, bif.bk_get_config('orbits_observe'))
        self.setValueSilent(self.numOfSkipOrbitsSpinbox, bif.bk_get_config('orbits_skip'))
        self.setValueSilent(self.countSpinbox, bif.bk_get_config('acquisition_count'))
        self.setValueSilent(self.waitSpinbox, bif.bk_get_config('orbits_wait_time'))
        self.tickSilent(self.buildSpectrogrammTickbox, bif.bk_get_config('build_spectrograms'))
        self.tickSilent(self.simulatePilotBunch, bif.bk_get_config('pilot_bunch'))

    # -----------------[ Value Change Handlers ]---------------------------
    def on_number_of_orbits_changed(self):
        """ Handle the change of the number of orbits"""
        bif.bk_change_num_of_orbits(self.numOfOrbitsSpinbox.value())
    def on_number_of_skipped_orbits_changed(self):
        """ Handle the change of the number of skipped orbits"""
        bif.bk_change_num_of_skipped_orbits(self.numOfSkipOrbitsSpinbox.value())
    def on_count_changed(self):
        """ Handle the change of the count value """
        bif.bk_change_count(self.countSpinbox.value())
    def on_wait_changed(self):
        """ Handle the change of the wait time """
        bif.bk_change_wait(self.waitSpinbox.value())
    def on_build_spectrograms_changed(self):
        """ Handle the change of the build_spectrograms checkbox"""
        bif.bk_change_build_spectrograms(self.buildSpectrogrammTickbox.checkState())
    def on_simulate_pilot_bunch_changed(self):
        """ Handle the change of the simulate_pilot_bunch checkbox"""
        bif.bk_change_pilot_bunch(self.simulatePilotBunch.checkState())
    def on_trigger_skip_signals_changed(self, value):
        """ Handle change of trigger skip signal spinbox """
        bif.bk_update_config('trigger_skip', value)
    def on_trigger_method_changed(self):
        """ Handle change of trigger method. """
        bif.bk_update_config('trigger_method', 2 if self.method.state() else 1)
    def on_trigger_timeout_changed(self, value):
        """ Handle change of trigger timeout spinbox """
        bif.bk_update_config('trigger_timeout', value)


    def closeEvent(self, event):
        """
        Event handler to handle the event of closing this window and gracefully delete resources and such
        """
        board.config.unobserve(self.numOfOrbitsSpinbox, 'orbits_observe')
        board.config.unobserve(self.numOfSkipOrbitsSpinbox, 'orbits_skip')
        board.config.unobserve(self.countSpinbox, 'acquisition_count')
        board.config.unobserve(self.waitSpinbox, 'orbits_wait_time')
        board.config.unobserve(self.simulatePilotBunch, 'pilot_bunch')
        board.config.unobserve(self.buildSpectrogrammTickbox, 'build_spectrograms')
        board.config.unobserve(self.trigger_skip_signals, 'trigger_skip')
        board.config.unobserve(self.timeout, 'trigger_timeout')
        board.config.unobserve(self.trigger_tick, 'use_trigger')
        Elements.removeItem(["acquire", "no_board"],
             [
                 self.numOfOrbitsSpinbox,
                 self.numOfSkipOrbitsSpinbox,
                 self.countSpinbox,
                 self.waitSpinbox,
                 self.buildSpectrogrammTickbox,
                 self.simulatePilotBunch,
                 self.trigger_skip_signals,
                 self.timeout,
                 self.method,
                 self.trigger_tick
             ])

        global __widget_id__
        __widget_id__ = None
        del self.par.widgets[self.id]
        super(acquireSettings, self).closeEvent(event)


def addAcquireSettingsWidgets():
    """
    Add this widget to the gui.
    This function will actually open the subwindow.
    :return: -
    """
    global __widget_id__
    if __widget_id__:
        kcgw.area.widgets[__widget_id__].setFocus()
    else:
        nid = kcgw.idg.genid()
        __widget_id__ = nid
        w = acquireSettings(nid, kcgw.area)
        kcgw.area.newWidget(w, "Acquire", nid, widget_type=4, minSize=True) #TODO: proper type

def setValueSilent(key, value):
    """
    Set values without notifying observers
    :param key: the key
    :param value: the value to set
    :return: -
    """
    if not __widget_id__:
        return
    kcgw.area.widgets[__widget_id__].blockSignals(True)
    if key == "orbits_observe":
        kcgw.area.widgets[__widget_id__].numOfOrbitsSpinbox.setValue(value)
    if key == "orbits_skip":
        kcgw.area.widgets[__widget_id__].numOfSkipOrbitsSpinbox.setValue(value)
    kcgw.area.widgets[__widget_id__].blockSignals(False)


kcgw.register_widget(QtGui.QIcon(config.install_path + config.acquireSettingsIcon), tr("Heading", "Acquire Settings"), addAcquireSettingsWidgets, "Ctrl+A")
