"""
This module contains the window to use an external Trigger on KAPTURE
"""

from PyQt4 import QtGui

from ..base import kcgwidget as kcgw
from ..base import backendinterface as bif
from ..base.groupedelements import Elements
from .. import config
tr = kcgw.tr

__widget_id__ = None

class externalTrigger(kcgw.KCGWidgets):
    def __init__(self, unique_id, parent=None):
        super(externalTrigger, self).__init__()

        self.id = unique_id
        self.par = parent

        self.layout = QtGui.QGridLayout()
        self.setLayout(self.layout)

        self.layout.addWidget(self.createLabel(tr("Label", "Number of acquisitions")), 0, 0)
        self.layout.addWidget(self.createLabel(tr("Label", "Number of trigger signals to skip")), 1, 0)
        self.layout.addWidget(self.createLabel(tr("Label", "Timeout")), 2, 0)
        self.layout.addWidget(self.createLabel(tr("Label", "Performed Acquisitions")), 3, 0)
        self.layout.addWidget(self.createLabel(tr("Label", "Wait Method")), 4, 0)

        self.number_of_acquisitions = self.createSpinbox(0, 10000000, 1, start_value=255)
        self.number_of_skip_signals = self.createSpinbox(0, 1000, 1, start_value=0)
        self.timeout = self.createSpinbox(0, 10000000, start_value=8)
        self.count_label = self.createLabel("0")
        self.method = kcgw.Switch("1", "2")

        self.layout.addWidget(self.number_of_acquisitions, 0, 1)
        self.layout.addWidget(self.number_of_skip_signals, 1, 1)
        self.layout.addWidget(self.timeout, 2, 1)
        self.layout.addWidget(self.count_label, 3, 1)
        self.layout.addWidget(self.method, 4, 1)

        self.toggle_wait_for_trigger_button = self.createButton(tr("Button", "Start waiting for trigger"),
                                 connect=self.toggle_waiting)

        self.layout.addWidget(self.toggle_wait_for_trigger_button, 5, 0)

        Elements.addItem(['acquire', 'wait_on_trigger'], self.toggle_wait_for_trigger_button)
        self.setWindowTitle(tr("Heading", "External Trigger"))

    def toggle_waiting(self):
        """
        Toggle waiting for trigger, this will write num of acquisitions and num of skip signals to board.
        :return:
        """
        method = 2 if self.method.state() else 1
        bif.bk_toggle_wait_on_trigger(self.number_of_acquisitions.value(), self.number_of_skip_signals.value(), self.count_label, self.timeout.value(), method)

    def closeEvent(self, event):
        """
        Event handler to handle the event of closing this window and gracefully delete ids and such
        """
        global __widget_id__
        __widget_id__ = None
        del self.par.widgets[self.id]
        Elements.removeItem(['acquire', 'wait_on_trigger'], self.toggle_wait_for_trigger_button)
        super(externalTrigger, self).closeEvent(event)


def addExternalTriggerWidget():
    """
    Add this widget to the gui.
    This function will actually open the subwindow.
    :return: -
    """
    global __widget_id__
    if __widget_id__:
        kcgw.area.widgets[__widget_id__].setFocus()
    else:
        nid = kcgw.idg.genid()
        __widget_id__ = nid
        w = externalTrigger(nid, kcgw.area)
        kcgw.area.newWidget(w, tr("Heading", "External Trigger"), nid, widget_type=4, minSize=True) #TODO: proper type

kcgw.register_widget(QtGui.QIcon(config.install_path + config.timingIcon), tr("Heading", "Trigger"), addExternalTriggerWidget, "Ctrl+E")