from PyQt4 import QtGui, QtCore
import pyqtgraph as pg
import pyqtgraph.exporters as exp
import os
import socket
from collections import OrderedDict
from ..base import kcgwidget as kcgw
from ..base import backendinterface as bif
from ..base.plotWidget import SubPlotWidget, PlotType
from ..base.backend import board
from ..base.groupedelements import Elements
from ..base.backend import io
from .. import config

__widget_id__ = None
DOES_NOT_SET_CLOSE = 1
DOES_SET_CLOSE = 2

class Remote(kcgw.KCGWidgets):
    response = QtCore.pyqtSignal(str)
    close_connection_requested = QtCore.pyqtSignal()
    def __init__(self, unique_id, parent):
        super(Remote, self).__init__()

        board.status.last_file = '/home/blaxxun/Documents/Hiwi/KaptureSimulator/2015-05-07_test-patrick/1431006860.291.out'

        self.id = unique_id
        self.par = parent

        self.status = False
        self.send_status = False
        self.host = '0.0.0.0'

        self.layout = QtGui.QGridLayout()
        self.setLayout(self.layout)
        self.start_stop_button_listen = self.createButton("Allow Remote Control", connect=self.start_stop_listen)
        self.start_stop_button_send = self.createButton("Control remote Gui", connect=self.start_stop_send)
        self.port = self.createSpinbox(1024, 65000, interval=1, start_value=1330)
        self.port_label = self.createLabel("Port")
        self.layout.addWidget(self.port_label, 0, 0)
        self.layout.addWidget(self.port, 0, 1)
        self.layout.addWidget(self.start_stop_button_listen, 1, 1)

        self.host_label = self.createLabel("Host")
        self.host_input = self.createInput()
        self.layout.addWidget(self.host_label, 2, 0)
        self.layout.addWidget(self.host_input, 2, 1)
        self.layout.addWidget(self.port_label, 3, 0)
        self.layout.addWidget(self.port, 3, 1)
        self.layout.addWidget(self.start_stop_button_send, 4, 1)

        self.rl = RemoteListener(self.host, self.port.value(), parent=self)
        self.rl.command_received.connect(self.evaluate)
        # self.response.connect(self.rl.response)
        self.response.connect(self.rl.responde)

        self.commands = {
            'calibrate': [["after_start"], bif.bk_calibrate, DOES_NOT_SET_CLOSE],
            'sync': [["after_start", "synchronize"], bif.bk_sync_board, DOES_NOT_SET_CLOSE],
            'set_defaults': [['after_start', 'set_defaults'], lambda: bif.bk_write_values(defaults=True), DOES_NOT_SET_CLOSE],
            'soft_reset': [['after_start'], bif.bk_soft_reset, DOES_NOT_SET_CLOSE],
            'acquire': [[], self.acquire, DOES_SET_CLOSE],
            'set': [[], self.set_value, DOES_SET_CLOSE],
            'get': [[], self.get_value, DOES_SET_CLOSE],
            'help': [[], self.help, DOES_SET_CLOSE],
            'get_image': [[], self.get_image, DOES_SET_CLOSE],
            'singleread': [['acquire'], self.single_read, DOES_SET_CLOSE]
        }

        self.value_types = OrderedDict({
            'fpga_delay_max': int,
            'fpga_delay': int,
            'fpga_delay_factor': int,
            'chip_delay_max': int,
            'chip_1_delay': int,
            'chip_2_delay': int,
            'chip_3_delay': int,
            'chip_4_delay': int,
            'chip_delay_factor': int,
            'th_delay_max': int,
            'th_delay': int,
            'th_delay_factor': int,
            'adc_delay_max': int,
            'adc_1_delay': int,
            'adc_2_delay': int,
            'adc_3_delay': int,
            'adc_4_delay': int,
            'adc_delay_factor': int,
            'th_to_adc_cycles': int,
            'adc_1_delay_individual': int,
            'orbits_observe': int,
            'orbits_skip': int,
            'acquisition_count': int,
            'orbits_wait_time': int,
            'build_spectrograms': bool,
            'pilot_bunch': bool,
            'header': bool
        })
        self.help_text = {
            'calibrate': ['-', 'Calibrates the board'],
            'sync': ['-', 'Synchronizes the board'],
            'set_defaults': ['-', 'Sets defaults and writes them to the board'],
            'soft_reset': ['-', 'Performs a soft reset'],
            'aquire': ['-', 'Starts an acquisition'],
            'set': [["One of", self.value_types.keys(), '<value>'], 'Write values'],
            'get': [['Sublist of', self.value_types.keys()], 'Read values'],
            'help': [['[command]'], 'Get Help'],
            'get_image': [['<type>', '<adc>'], 'Gets the last acquired data as image (type is one of "trains", "fft", "heatmap", "combined"'],
            'singleread': [[], 'Performs a single read']
        }


    def is_possible(self, groups):
        for group in groups:
            print group
            if not Elements.isEnabled(group):
                return False
        return True

    # -----[ remote commands ]----------
    def acquire(self):
        if self.is_possible(["acquireTrigger"]):
            bif.bk_acquire()
            if board.status.acquisition:
                self.response.emit("Acquisition started - Check status using 'get status'")
            else:
                self.response.emit("Acquisition stopped")
        else:
            self.response.emit("Not Possible")

    def single_read(self):
        try:
            bif.bk_single_read()
            self.response.emit("Singleread done")
        except Exception as e:
            self.response.emit(str(e))

    def set_value(self, c):
        if len(c) != 2:
            self.response.emit("You need to specify the option (only one) and a value.\nset <option> <value>")
            return
        try:
            board.config.update(c[0], self.value_types[c[0]](c[1]))
            self.response.emit("Success: Set Value "+c[0]+" to "+c[1])
        except (ValueError, KeyError) as e:
            self.response.emit("Failed - Invalid Value: " + str(e))

    def get_value(self, c):
        if c[0] == 'status':
            if board.status.acquisition:
                perc = int(board.acquisition_progressbar.value() / float(
                                                             board.acquisition_progressbar.maximum() -
                                                             board.acquisition_progressbar.minimum()) * 100)
                perc = str(perc) + "%"
            else:
                perc = "No Acquisition"
            self.response.emit(str(perc))
        elif c[0] == 'all_acquisition': # get all values important for acquisition
            resp = "Orbits Observed: " + str(board.config.get("orbits_observe")) + "\n" + \
                   "Orbits Skipped: " + str(board.config.get("orbits_skip")) + "\n" + \
                   "Count: " + str(board.config.get("acquisition_count")) + "\n" + \
                   "Wait Time: " + str(board.config.get("orbits_wait_time")) + "\n" + \
                   "Pilot Bunch Simulator: " + str(board.config.get("pilot_bunch")) + "\n" + \
                   "Header: " + str(board.config.get("header"))
            self.response.emit(resp)
        elif c[0] == 'possible_keys':
            resp = ''
            for key in self.value_types.keys():
                resp += key + '\n'
            self.response.emit(resp)
        else:
            if len(c) == 2 and c[1] == 'clean':
                try:
                    self.response.emit(str(board.config.get(c[0])))
                except ValueError as e:
                    self.response.emit("Failed - Invalid Value: " + str(e))
            else:
                ret = ""
                try:
                    for key in c:
                        ret += key + " = " + str(board.config.get(key)) + "\n"
                    self.response.emit(ret[:-1])
                except ValueError as e:
                    self.response.emit("Failed - Invalid Value: " + str(e))

    def help(self, c=None):
        if isinstance(c, list):
            command = c[0]
        if c is None:
            ret = "Possible commands:"
            for key in self.commands:
                ret += '\n'+ key
        elif command in self.help_text:
            ret = command + ":\n"
            ret += "Options: "
            if isinstance(self.help_text[command][0], list):
                for i in self.help_text[command][0]:
                    ret += str(i) + ", "
            else:
                ret += str(self.help_text[command][0])
            ret += '\n'
            ret += str(self.help_text[command][1])
        else:
            ret = "Either the command or the help for this command doesn't exist"
        self.response.emit(ret)

    def get_image(self, c=None):
        if c is None:
            c = ['trains', 1]
        imgpath = board.status.last_file
        sw = SubPlotWidget()
        if c[0] == 'trains':
            data = io.read_from_file(imgpath, force=False, header=board.config.get('header'), cache=False).train(adc=int(c[1]), frm=0, to=10000)
            # test = pg.plot(data)
            # export = exp.ImageExporter(test.plotItem)
            # export.parameters()['width'] = 500
            sw.changeType(PlotType.Trains)
            # sw.plot(data)
            # export.export('tempimage.png')
            # test.parent().close()
            # del test
            # del export
        elif c[0] == 'fft':
            data = io.read_from_file(imgpath, force=False, header=board.config.get('header'), cache=False).fft(adc=int(c[1]), frm=0, to=10000, drop_first_bin=True)
            sw.changeType(PlotType.FFT)
        elif c[0] == 'heatmap':
            data = io.read_from_file(imgpath, force=False, header=board.config.get('header'), cache=False).heatmap(adc=int(c[1]), frm=0, to=10000)
            sw.changeType(PlotType.Heatmap)
        elif c[0] == 'combined':
            data = io.read_from_file(imgpath, force=False, header=board.config.get('header'), cache=False).combined(frm=0, to=10000)
            sw.changeType(PlotType.Combined)
        sw.plot(data, autorange=True)
        QtGui.QApplication.processEvents()
        export = exp.ImageExporter(sw.plotItem)
        export.parameters()['width'] = 1000
        export.export('tempimage.png')
        f = open('tempimage.png', 'r')
        self.rl.c.sendall(f.read())
        f.close()
        os.remove('tempimage.png')
        sw.close()
        del sw
        del export


    @QtCore.pyqtSlot(str)
    def evaluate(self, command):
        c = str(command).split(" ")
        try:
            if c[0] in self.commands:
                if self.is_possible(self.commands[c[0]][0]):
                    # self.response.emit("Command Received")
                    if len(c) > 1:
                        self.commands[c[0]][1](c[1:])
                    else:
                        self.commands[c[0]][1]()
                    if self.commands[c[0]][2] == DOES_NOT_SET_CLOSE:
                        self.response.emit("Success: " + c[0])
                    self.close_connection_requested.emit()
                else:
                    self.response.emit("Not Possible")
        except Exception as e:
            self.response.emit("Failed \n"+str(e))


    def start_stop_listen(self):
        self.status = not self.status
        self.rl.status = self.status
        if not self.status:
            self.listen_process.terminate()
            self.start_stop_button_listen.setText("Allow Remote Control")
            del self.listen_process
        else:
            # self.listen_process = mp.Process(target=self.rl.listener)
            self.listen_process = QtCore.QThread()
            self.rl.moveToThread(self.listen_process)
            self.listen_process.started.connect(self.rl.listener)
            self.listen_process.start()
            self.start_stop_button_listen.setText("Disallow Remote Control")

    def start_stop_send(self):
        self.send_status = not self.send_status
        if not self.send_status:
            board.safe_call = board.backup_safe_call
            self.start_stop_button_send.setText("Control remote Gui")
        else:
            board.host = self.host_input.text()
            board.port = self.port.value()
            board.safe_call = board.safe_remote_call
            Elements.setEnabled('no_board', True)
            bif.bk_check_for_board()
            bif.bk_status_readout()
            self.start_stop_button_send.setText("Control local Gui")

    def closeEvent(self, event):
        global __widget_id__
        __widget_id__ = None
        del self.par.widgets[self.id]

def addRemoteWidget():
    global __widget_id__
    if __widget_id__:
        kcgw.area.widgets[__widget_id__].setFocus()
    else:
        nid = kcgw.idg.genid()
        __widget_id__ = nid
        w = Remote(nid, kcgw.area)
        kcgw.area.newWidget(w, "Remote Widget", nid, widget_type=4, minSize=True)

kcgw.register_widget(QtGui.QIcon(config.install_path + "icons/project.svg"), "Remote Widget", addRemoteWidget, "Ctrl+r")

class RemoteListener(QtCore.QObject):
    command_received = QtCore.pyqtSignal(str)

    def __init__(self, host, port, parent=None):
        super(RemoteListener, self).__init__()
        self.host = host
        self.port = port
        self.parent = parent
        self.status = True
        self.soc = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        self.soc.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
        self.soc.bind((self.host, self.port))
        self.soc.listen(0)
        self.response_text = None
        self.response_updated = False

    def evaluate(self, command):
        self.command_received.emit(command)

    def listener(self):

        # conn, addr = soc.accept()
        while self.status:
            try:
                self.close = False
                conn, addr = self.soc.accept()
                self.c = conn
                command = conn.recv(1024)
                if not command: continue
                # if command == "stop":
                #     break
                self.evaluate(command)

                # -------[ OLD wait ]---------
                # self.wait_signal(self.parent.response, 20000)
                # if self.response_text:
                    # while self.response_text[0] == '.':
                    #     conn.send(self.response_text)
                    #     self.wait_signal(self.parent.response, 20000)
                    # conn.send(self.response_text)
                # else:
                #     conn.send("Failed")

                # self.response_text = None
                self.wait_signal(self.parent.close_connection_requested, 10000)
                conn.send("close") # tell the client to close the connection
                conn.close()
            except socket.error:  # Go on even in case of exception
                pass
        self.c.send("close") # tell the client to close the connection
        self.c.close()

    def response(self, res):
        self.response_text = res
        self.response_updated = True

    def responde(self, res):
        try:
            self.c.sendall(str(res))
        except Exception as e:
            print str(e)

    def wait_signal(self, signal, timeout=10000):
        """Block loop until signal emitted, or timeout (ms) elapses."""
        loop = QtCore.QEventLoop()
        signal.connect(loop.quit)

        if timeout is not None:
            QtCore.QTimer.singleShot(timeout, loop.quit)
        loop.exec_()

