# -*- coding: utf-8 -*-
"""
This Module is the SingleRead window.
"""
from PyQt4 import QtGui

from ..base import kcgwidget as kcgw
from ..base import backendinterface as bif
from ..base.groupedelements import Elements
from ..base.storage import storage
from .. import config

tr = kcgw.tr

__widget_id__ = None

class singleReadWidget(kcgw.KCGWidgets):
    """
    The actual single read window.
    """
    def __init__(self, unique_id, parent=None):
        """
        Initialise the single read widget
        :param unique_id: the id for this widget
        :param parent: parent object
        :return: -
        """
        super(singleReadWidget, self).__init__()

        self.id = unique_id
        self.par = parent
        self.layout = QtGui.QGridLayout()
        self.outerLayout = QtGui.QVBoxLayout()
        self.outerLayout.addLayout(self.layout)
        self.outerLayout.addStretch(1)
        self.setLayout(self.outerLayout)
        self.continuous_read = False

        #---------[ Create Labels and corresponding Fields ]---------
        self.single_read_button = self.createButton(tr("Button", "Single Read"), connect=bif.bk_single_read)
        self.continuous_read_button = self.createButton(tr("Button", "Start Continuous Read"), connect=self.on_continuous_read)
        self.interval_spinbox = self.createSpinbox(0, 100000, start_value=1000, connect=self.set_interval)
        Elements.addItem(["acquire", "no_board"], [self.single_read_button, self.continuous_read_button])
        Elements.addItem("continuous_read", self.continuous_read_button)

        #--------[ Fill Grid ]----------------
        self.layout.addWidget(self.single_read_button, 0, 0)
        self.layout.addWidget(self.continuous_read_button, 0, 1)
        self.layout.addWidget(self.createLabel(tr("Label", "Interval:") + " (ms)"), 1, 0)
        self.layout.addWidget(self.interval_spinbox, 1, 1)
        self.setWindowTitle(tr("Heading", "Single and Continuous Read"))

        self.set_interval()  # initially set interval

    def on_continuous_read(self):
        """
        Toggle function to toggle continuous read when the corresponding button is pressed.
        :return: -
        """
        if not self.continuous_read:
            self.continuous_read_button.setText(tr("Button", "Stop Continuous Read"))
            self.continuous_read = True
        else:
            self.continuous_read_button.setText(tr("Button", "Start Continuous Read"))
            self.continuous_read = False
        bif.bk_continuous_read() # interval=self.interval_spinbox.value())

    def set_interval(self):
        storage.continuous_interval = self.interval_spinbox.value()

    def closeEvent(self, event):
        """
        Event handler to handle the event of closing this window and gracefully delete ids and such
        """
        global __widget_id__
        __widget_id__ = None
        del self.par.widgets[self.id]
        Elements.removeItem(['no_board', 'acquire'],
                            [
                                self.single_read_button,
                                self.continuous_read_button
                            ])

        super(singleReadWidget, self).closeEvent(event)


def addSingleReadWidget():
    """
    Add this widget to the gui.
    This function will actually open the subwindow.
    :return: -
    """
    global __widget_id__
    if __widget_id__:
        kcgw.area.widgets[__widget_id__].setFocus()
    else:
        nid = kcgw.idg.genid()
        __widget_id__ = nid
        w = singleReadWidget(nid, kcgw.area)
        kcgw.area.newWidget(w, "Single and Continuous Read", nid, widget_type=4, minSize=True) #TODO: proper type

kcgw.register_widget(QtGui.QIcon(config.install_path + config.singleReadIcon), tr("Heading", "Single Read"), addSingleReadWidget, "Ctrl+i")
